@@ -1,3 +1,13 @@
+/*
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Copyright (C) 2014-2023 Karel Zak <kzak@redhat.com>
+ */
 #ifndef UTIL_LINUX_FDISK_H
 #define UTIL_LINUX_FDISK_H
 /*
@@ -25,13 +35,15 @@
 #define FDISKPROG_DEBUG_ALL	0xFFFF
 
 extern int pwipemode;
+extern struct fdisk_table *original_layout;
+extern int device_is_used;
+extern int is_interactive;
 
 UL_DEBUG_DECLARE_MASK(fdisk);
 #define DBG(m, x)       __UL_DBG(fdisk, FDISKPROG_DEBUG_, m, x)
 #define ON_DBG(m, x)    __UL_DBG_CALL(fdisk, FDISKPROG_DEBUG_, m, x)
 
-extern int get_user_reply(struct fdisk_context *cxt,
-			  const char *prompt,
+extern int get_user_reply(const char *prompt,
 			  char *buf, size_t bufsz);
 extern int process_fdisk_menu(struct fdisk_context **cxt);
 
@@ -49,4 +61,6 @@ extern void change_partition_type(struct fdisk_context *cxt);
 
 extern void toggle_dos_compatibility_flag(struct fdisk_context *cxt);
 
+extern void follow_wipe_mode(struct fdisk_context *cxt);
+
 #endif /* UTIL_LINUX_FDISK_H */
