@@ -1,11 +1,17 @@
 /*
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
  * fsck.minix.c - a file system consistency checker for Linux.
  *
  * (C) 1991, 1992 Linus Torvalds. This file may be redistributed
  * as per the GNU copyleft.
- */
-
-/*
+ *
+ *
  * 09.11.91  -  made the first rudimentary functions
  *
  * 10.11.91  -  updated, does checking, no repairs yet.
@@ -293,7 +299,6 @@ check_mount(void) {
 		printf(_("check aborted.\n"));
 		exit(FSCK_EX_OK);
 	}
-	return;
 }
 
 
@@ -301,7 +306,7 @@ static int is_valid_zone_nr(unsigned short nr)
 {
 	if (nr < get_first_zone())
 		return 0;
-	else if (nr >= get_nzones())
+	if (nr >= get_nzones())
 		return 0;
 	return 1;
 }
@@ -511,7 +516,6 @@ write_super_block(void) {
 		die(_("seek failed in write_super_block"));
 	if (MINIX_BLOCK_SIZE != write(device_fd, super_block_buffer, MINIX_BLOCK_SIZE))
 		die(_("unable to write super-block"));
-	return;
 }
 
 static void
@@ -978,7 +982,7 @@ check_file(struct minix_inode *dir, unsigned int offset) {
 	inode = get_inode(ino);
 	name_depth--;
 	if (!offset) {
-		if (!inode || strcmp(".", name)) {
+		if (!inode || strcmp(".", name) != 0) {
 			get_current_name();
 			printf(_("%s: bad directory: '.' isn't first\n"),
 			       current_name);
@@ -987,7 +991,7 @@ check_file(struct minix_inode *dir, unsigned int offset) {
 			return;
 	}
 	if (offset == dirsize) {
-		if (!inode || strcmp("..", name)) {
+		if (!inode || strcmp("..", name) != 0) {
 			get_current_name();
 			printf(_("%s: bad directory: '..' isn't second\n"),
 			       current_name);
@@ -1017,7 +1021,6 @@ check_file(struct minix_inode *dir, unsigned int offset) {
 	if (inode && S_ISDIR(inode->i_mode))
 		recursive_check(ino);
 	name_depth--;
-	return;
 }
 
 static void
@@ -1052,7 +1055,7 @@ check_file2(struct minix2_inode *dir, unsigned int offset) {
 	inode = get_inode2(ino);
 	name_depth--;
 	if (!offset) {
-		if (!inode || strcmp(".", name)) {
+		if (!inode || strcmp(".", name) != 0) {
 			get_current_name();
 			printf(_("%s: bad directory: '.' isn't first\n"),
 			       current_name);
@@ -1061,7 +1064,7 @@ check_file2(struct minix2_inode *dir, unsigned int offset) {
 			return;
 	}
 	if (offset == dirsize) {
-		if (!inode || strcmp("..", name)) {
+		if (!inode || strcmp("..", name) != 0) {
 			get_current_name();
 			printf(_("%s: bad directory: '..' isn't second\n"),
 			       current_name);
@@ -1087,7 +1090,6 @@ check_file2(struct minix2_inode *dir, unsigned int offset) {
 	if (inode && S_ISDIR(inode->i_mode))
 		recursive_check2(ino);
 	name_depth--;
-	return;
 }
 
 static void
@@ -1359,7 +1361,9 @@ main(int argc, char **argv) {
 		if (repair)
 			printf(_("%s is clean, no check.\n"), device_name);
 		return retcode;
-	} else if (force)
+	}
+
+	if (force)
 		printf(_("Forcing filesystem check on %s.\n"), device_name);
 	else if (repair)
 		printf(_("Filesystem on %s is dirty, needs checking.\n"),
