@@ -1,5 +1,13 @@
 /*
- * isosize.c - Andries Brouwer, 000608
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Copyright (C) 2000 Andries Brouwer
+ * Copyright (C) 2023 Karel Zak <kzak@redhat.com>
  *
  * use header info to find size of iso9660 file system
  * output a number - useful in scripts
@@ -16,7 +24,6 @@
  *     - add "-d <num>" option and use long long to fix things > 2 GB
  *  Version 2.02 2000/10/11
  *     - error messages on IO failures [D. Gilbert]
- *
  */
 #include <stdio.h>
 #include <stdlib.h>
@@ -29,6 +36,7 @@
 #include "c.h"
 #include "strutils.h"
 #include "closestream.h"
+#include "iso9660.h"
 
 #define ISOSIZE_EXIT_ALLFAILED	32
 #define ISOSIZE_EXIT_SOMEOK	64
@@ -42,56 +50,6 @@ static int is_iso(int fd)
 	return memcmp(&label, &"\1CD001\1", 8);
 }
 
-static int isonum_721(unsigned char *p)
-{
-	return ((p[0] & 0xff)
-		| ((p[1] & 0xff) << 8));
-}
-
-static int isonum_722(unsigned char *p)
-{
-	return ((p[1] & 0xff)
-		| ((p[0] & 0xff) << 8));
-}
-
-static int isonum_723(unsigned char *p, int xflag)
-{
-	int le = isonum_721(p);
-	int be = isonum_722(p + 2);
-
-	if (xflag && le != be)
-		/* translation is useless */
-		warnx("723error: le=%d be=%d", le, be);
-	return (le);
-}
-
-static int isonum_731(unsigned char *p)
-{
-	return ((p[0] & 0xff)
-		| ((p[1] & 0xff) << 8)
-		| ((p[2] & 0xff) << 16)
-		| ((p[3] & 0xff) << 24));
-}
-
-static int isonum_732(unsigned char *p)
-{
-	return ((p[3] & 0xff)
-		| ((p[2] & 0xff) << 8)
-		| ((p[1] & 0xff) << 16)
-		| ((p[0] & 0xff) << 24));
-}
-
-static int isonum_733(unsigned char *p, int xflag)
-{
-	int le = isonum_731(p);
-	int be = isonum_732(p + 4);
-
-	if (xflag && le != be)
-		/* translation is useless */
-		warnx("733error: le=%d be=%d", le, be);
-	return (le);
-}
-
 static int isosize(int argc, char *filenamep, int xflag, long divisor)
 {
 	int fd, nsecs, ssize, rc = -1;
