@@ -274,3 +274,15 @@ DEF_GUID("42465331-3BA3-10F1-802A-4861696B7521", N_("Haiku BFS")),
 
 /* Marvell */
 DEF_GUID("6828311A-BA55-42A4-BCDE-A89BB5EDECAE", N_("Marvell Armada 3700 Boot partition")),
+
+/* DragonFly BSD */
+/* https://github.com/DragonFlyBSD/DragonFlyBSD/blob/master/etc/defaults/uuids */
+DEF_GUID("9D087404-1CA5-11DC-8817-01301BB8A9F5", N_("DragonFlyBSD Label32")),
+DEF_GUID("9D58FDBD-1CA5-11DC-8817-01301BB8A9F5", N_("DragonFlyBSD Swap")),
+DEF_GUID("9D94CE7C-1CA5-11DC-8817-01301BB8A9F5", N_("DragonFlyBSD UFS1")),
+DEF_GUID("9DD4478F-1CA5-11DC-8817-01301BB8A9F5", N_("DragonFlyBSD Vinum")),
+DEF_GUID("DBD5211B-1CA5-11DC-8817-01301BB8A9F5", N_("DragonFlyBSD CCD")),
+DEF_GUID("3D48CE54-1D16-11DC-8696-01301BB8A9F5", N_("DragonFlyBSD Label64")),
+DEF_GUID("BD215AB2-1D16-11DC-8696-01301BB8A9F5", N_("DragonFlyBSD Legacy")),
+DEF_GUID("61DC63AC-6E38-11DC-8513-01301BB8A9F5", N_("DragonFlyBSD HAMMER")),
+DEF_GUID("5CBB9AD1-862D-11DC-A94D-01301BB8A9F5", N_("DragonFlyBSD HAMMER2")),
