@@ -1,3 +1,7 @@
+/*
+ * No copyright is claimed.  This code is in the public domain; do with
+ * it what you wish.
+ */
 #ifndef UTIL_LINUX_PT_MBR_H
 #define UTIL_LINUX_PT_MBR_H
 
@@ -58,6 +62,28 @@ static inline void dos_partition_set_size(struct dos_partition *p, unsigned int
 	__dos_store_4le(p->nr_sects, n);
 }
 
+static inline void dos_partition_sync_chs(struct dos_partition *p, unsigned long long int part_offset, unsigned int geom_sectors, unsigned int geom_heads)
+{
+	unsigned long long int start = part_offset + dos_partition_get_start(p);
+	unsigned long long int stop = start + dos_partition_get_size(p) - 1;
+	unsigned int spc = geom_heads * geom_sectors;
+
+	if (start / spc > 1023)
+		start = spc * 1024 - 1;
+	if (stop / spc > 1023)
+		stop = spc * 1024 - 1;
+
+	p->bc = (start / spc) & 0xff;
+	p->bh = (start / geom_sectors) % geom_heads;
+	p->bs = ((start % geom_sectors + 1) & 0x3f) |
+		(((start / spc) >> 2) & 0xc0);
+
+	p->ec = (stop / spc) & 0xff;
+	p->eh = (stop / geom_sectors) % geom_heads;
+	p->es = ((stop % geom_sectors + 1) & 0x3f) |
+		(((stop / spc) >> 2) & 0xc0);
+}
+
 static inline int mbr_is_valid_magic(const unsigned char *mbr)
 {
 	return mbr[510] == 0x55 && mbr[511] == 0xaa ? 1 : 0;
@@ -177,6 +203,7 @@ enum {
 	MBR_SPEEDSTOR1_PARTITION	= 0xf1,
 	MBR_SPEEDSTOR2_PARTITION	= 0xf4, /* SpeedStor large partition */
 	MBR_DOS_SECONDARY_PARTITION	= 0xf2, /* DOS 3.3+ secondary */
+	MBR_EBBR_PROTECTIVE_PARTITION	= 0xf8, /* Arm EBBR firmware protective partition */
 	MBR_VMWARE_VMFS_PARTITION	= 0xfb,
 	MBR_VMWARE_VMKCORE_PARTITION	= 0xfc, /* VMware kernel dump partition */
 	MBR_LINUX_RAID_PARTITION	= 0xfd, /* Linux raid partition with autodetect using persistent superblock */
