@@ -1,3 +1,7 @@
+/*
+ * No copyright is claimed.  This code is in the public domain; do with
+ * it what you wish.
+ */
 #ifndef UTIL_LINUX_STRUTILS
 #define UTIL_LINUX_STRUTILS
 
@@ -42,11 +46,14 @@ extern uint64_t str2unum_or_err(const char *str, int base, const char *errmesg,
 extern double strtod_or_err(const char *str, const char *errmesg);
 extern long double strtold_or_err(const char *str, const char *errmesg);
 
-extern long strtol_or_err(const char *str, const char *errmesg);
-extern unsigned long strtoul_or_err(const char *str, const char *errmesg);
+#define strtol_or_err(_s, _e)	(long) str2num_or_err(_s, 10, _e, LONG_MIN, LONG_MAX)
+#define strtopid_or_err(_s, _e)	(pid_t) str2num_or_err(_s, 10, _e, 1, SINT_MAX(pid_t))
+#define strtoul_or_err(_s, _e)	(unsigned long) str2unum_or_err(_s, 10, _e, ULONG_MAX)
 
 extern void strtotimeval_or_err(const char *str, struct timeval *tv,
 		const char *errmesg);
+extern void strtotimespec_or_err(const char *str, struct timespec *ts,
+		const char *errmesg);
 extern time_t strtotime_or_err(const char *str, const char *errmesg);
 
 extern int isdigit_strend(const char *str, const char **end);
@@ -211,7 +218,8 @@ extern int string_add_to_idarray(const char *list, int ary[],
 				 int (name2id)(const char *, size_t));
 
 extern int string_to_bitarray(const char *list, char *ary,
-			    int (*name2bit)(const char *, size_t));
+			    int (*name2bit)(const char *, size_t),
+			    size_t allow_range);
 
 extern int string_to_bitmask(const char *list,
 			     unsigned long *mask,
@@ -394,4 +402,6 @@ extern char *ul_strchr_escaped(const char *s, int c);
 extern int skip_fline(FILE *fp);
 extern int ul_stralnumcmp(const char *p1, const char *p2);
 
+extern int ul_optstr_next(char **optstr, char **name, size_t *namesz, char **value, size_t *valsz);
+
 #endif
