@@ -1,5 +1,8 @@
 /*
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ *
  * Copyright (C) 2010 Davidlohr Bueso <dave@gnu.org>
+ * Copyright (C) 2010-2022 Karel Zak <kzak@redhat.com>
  *
  * This file may be redistributed under the terms of the
  * GNU Lesser General Public License.
@@ -19,15 +22,6 @@
 # define XALLOC_EXIT_CODE EXIT_FAILURE
 #endif
 
-static inline
-__attribute__((__noreturn__))
-void __err_oom(const char *file, unsigned int line)
-{
-	err(XALLOC_EXIT_CODE, "%s: %u: cannot allocate memory", file, line);
-}
-
-#define err_oom()	__err_oom(__FILE__, __LINE__)
-
 static inline
 __ul_alloc_size(1)
 __ul_returns_nonnull
