@@ -1,6 +1,9 @@
 /*
  * Please, don't add this file to libcommon because clock_gettime() requires
  * -lrt on systems with old libc.
+ *
+ * No copyright is claimed.  This code is in the public domain; do with
+ * it what you wish.
  */
 #include <time.h>
 #include <signal.h>
@@ -45,6 +48,18 @@ int get_boot_time(struct timeval *boot_time)
 #endif
 }
 
+usec_t get_suspended_time(void)
+{
+#if defined(CLOCK_BOOTTIME) && defined(CLOCK_MONOTONIC)
+	struct timespec boot, mono;
+
+	if (clock_gettime(CLOCK_BOOTTIME, &boot) == 0 &&
+	    clock_gettime(CLOCK_MONOTONIC, &mono) == 0)
+		return timespec_to_usec(&boot) - timespec_to_usec(&mono);
+#endif
+	return 0;
+}
+
 int gettime_monotonic(struct timeval *tv)
 {
 #ifdef CLOCK_MONOTONIC
