@@ -33,7 +33,7 @@ int main(int argc, char *argv[])
 	devname = argv[1];
 	pr = blkid_new_probe_from_filename(devname);
 	if (!pr)
-		err(EXIT_FAILURE, "%s: faild to create a new libblkid probe",
+		err(EXIT_FAILURE, "%s: failed to create a new libblkid probe",
 				devname);
 
 	/*
@@ -54,7 +54,7 @@ int main(int argc, char *argv[])
 					"%s superblock", devname, type);
 
 		if (!blkid_probe_lookup_value(pr, "PTTYPE", &type, NULL))
-			errx(EXIT_FAILURE, "%s: appears to contain an partition "
+			errx(EXIT_FAILURE, "%s: appears to contain a partition "
 					"table (%s)", devname, type);
 	}
 
