@@ -237,6 +237,7 @@ struct blkid_struct_probe
 #define BLKID_FL_CDROM_DEV	(1 << 3)	/* is a CD/DVD drive */
 #define BLKID_FL_NOSCAN_DEV	(1 << 4)	/* do not scan this device */
 #define BLKID_FL_MODIF_BUFF	(1 << 5)	/* cached buffers has been modified */
+#define BLKID_FL_OPAL_LOCKED	(1 << 6)	/* OPAL device is locked (I/O errors) */
 
 /* private per-probing flags */
 #define BLKID_PROBE_FL_IGNORE_PT (1 << 1)	/* ignore partition table */
@@ -313,9 +314,6 @@ struct blkid_struct_cache
 /* old systems */
 #define BLKID_CACHE_FILE_OLD	"/etc/blkid.tab"
 
-#define BLKID_PROBE_OK	 0
-#define BLKID_PROBE_NONE 1
-
 #define BLKID_ERR_IO	 5
 #define BLKID_ERR_SYSFS	 9
 #define BLKID_ERR_MEM	12
@@ -410,6 +408,9 @@ extern int blkid_probe_is_tiny(blkid_probe pr)
 extern int blkid_probe_is_cdrom(blkid_probe pr)
 			__attribute__((nonnull))
 			__attribute__((warn_unused_result));
+extern int blkdid_probe_is_opal_locked(blkid_probe pr)
+			__attribute__((nonnull))
+			__attribute__((warn_unused_result));
 
 extern unsigned char *blkid_probe_get_buffer(blkid_probe pr,
                                 uint64_t off, uint64_t len)
@@ -433,9 +434,9 @@ extern int blkid_probe_get_idmag(blkid_probe pr, const struct blkid_idinfo *id,
 			__attribute__((nonnull(1)));
 
 /* returns superblock according to 'struct blkid_idmag' */
-extern unsigned char *_blkid_probe_get_sb(blkid_probe pr, const struct blkid_idmag *mag, size_t size);
+extern unsigned char *blkid_probe_get_sb_buffer(blkid_probe pr, const struct blkid_idmag *mag, size_t size);
 #define blkid_probe_get_sb(_pr, _mag, type) \
-			((type *) _blkid_probe_get_sb((_pr), _mag, sizeof(type)))
+			((type *) blkid_probe_get_sb_buffer((_pr), _mag, sizeof(type)))
 
 extern blkid_partlist blkid_probe_get_partlist(blkid_probe pr)
 			__attribute__((nonnull))
@@ -517,6 +518,8 @@ extern int blkid_probe_set_magic(blkid_probe pr, uint64_t offset,
 
 extern int blkid_probe_verify_csum(blkid_probe pr, uint64_t csum, uint64_t expected)
 			__attribute__((nonnull));
+extern int blkid_probe_verify_csum_buf(blkid_probe pr, size_t n, const void *csum,
+		const void *expected) __attribute__((nonnull));
 
 extern void blkid_unparse_uuid(const unsigned char *uuid, char *str, size_t len)
 			__attribute__((nonnull));
@@ -541,6 +544,9 @@ extern int blkid_probe_get_hint(blkid_probe pr, const char *name, uint64_t *valu
 			__attribute__((nonnull(1,2)))
 			__attribute__((warn_unused_result));
 
+extern int blkid_probe_get_partitions_flags(blkid_probe pr)
+			__attribute__((nonnull));
+
 /* filter bitmap macros */
 #define blkid_bmp_wordsize		(8 * sizeof(unsigned long))
 #define blkid_bmp_idx_bit(item)		(1UL << ((item) % blkid_bmp_wordsize))
