@@ -23,7 +23,7 @@
 /*
  * Sysfs topology values (since 2.6.31, May 2009).
  */
-static struct topology_val {
+static const struct topology_val {
 
 	/* /sys/dev/block/<maj>:<min>/<ATTR> */
 	const char *attr;
@@ -31,6 +31,7 @@ static struct topology_val {
 	/* functions to set probing result */
 	int (*set_ulong)(blkid_probe, unsigned long);
 	int (*set_int)(blkid_probe, int);
+	int (*set_u64)(blkid_probe, uint64_t);
 
 } topology_vals[] = {
 	{ "alignment_offset", NULL, blkid_topology_set_alignment_offset },
@@ -38,6 +39,7 @@ static struct topology_val {
 	{ "queue/optimal_io_size", blkid_topology_set_optimal_io_size },
 	{ "queue/physical_block_size", blkid_topology_set_physical_sector_size },
 	{ "queue/dax", blkid_topology_set_dax },
+	{ "diskseq", .set_u64 = blkid_topology_set_diskseq },
 };
 
 static int probe_sysfs_tp(blkid_probe pr,
@@ -58,7 +60,7 @@ static int probe_sysfs_tp(blkid_probe pr,
 	rc = 1;		/* nothing (default) */
 
 	for (i = 0; i < ARRAY_SIZE(topology_vals); i++) {
-		struct topology_val *val = &topology_vals[i];
+		const struct topology_val *val = &topology_vals[i];
 		int ok = ul_path_access(pc, F_OK, val->attr) == 0;
 
 		rc = 1;	/* nothing */
@@ -100,6 +102,12 @@ static int probe_sysfs_tp(blkid_probe pr,
 			if (ul_path_read_s64(pc, &data, val->attr) != 0)
 				continue;
 			rc = val->set_int(pr, (int) data);
+		} else if (val->set_u64) {
+			uint64_t data;
+
+			if (ul_path_read_u64(pc, &data, val->attr) != 0)
+				continue;
+			rc = val->set_u64(pr, data);
 		}
 
 		if (rc < 0)
