@@ -29,6 +29,7 @@ static int read_from_device(struct fdisk_context *cxt,
 		return -errno;
 	}
 
+	errno = 0;
 	r = read(cxt->dev_fd, buf, size);
 	if (r < 0 || (size_t)r != size) {
 		if (!errno)
@@ -142,7 +143,7 @@ char *fdisk_partname(const char *dev, size_t partno)
 
 	/* devfs kludge - note: fdisk partition names are not supposed
 	   to equal kernel names, so there is no reason to do this */
-	if (strcmp(dev + w - 4, "disc") == 0) {
+	if (endswith(dev, "disc")) {
 		w -= 4;
 		p = "part";
 	}
