@@ -13,7 +13,7 @@
 /**
  * SECTION: fs
  * @title: Filesystem
- * @short_description: represents one entry from fstab, mtab, or mountinfo file
+ * @short_description: represents one entry from fstab, or mountinfo file
  *
  */
 #include <ctype.h>
@@ -34,6 +34,7 @@
 struct libmnt_fs *mnt_new_fs(void)
 {
 	struct libmnt_fs *fs = calloc(1, sizeof(*fs));
+
 	if (!fs)
 		return NULL;
 
@@ -95,6 +96,11 @@ void mnt_reset_fs(struct libmnt_fs *fs)
 	free(fs->opt_fields);
 	free(fs->comment);
 
+	mnt_unref_optlist(fs->optlist);
+	fs->optlist = NULL;
+
+	fs->opts_age = 0;
+
 	memset(fs, 0, sizeof(*fs));
 	INIT_LIST_HEAD(&fs->ents);
 	fs->refcount = ref;
@@ -166,6 +172,73 @@ static inline int cpy_str_at_offset(void *new, const void *old, size_t offset)
 	return update_str(n, *o);
 }
 
+static inline int sync_opts_from_optlist(struct libmnt_fs *fs, struct libmnt_optlist *ol)
+{
+	unsigned int age;
+
+	assert(fs);
+	assert(ol);
+
+	age = mnt_optlist_get_age(ol);
+	if (age != fs->opts_age) {
+		const char *p;
+		int rc;
+
+		/* All options */
+		rc = mnt_optlist_get_optstr(ol, &p, NULL, 0);
+		if (!rc)
+			rc = strdup_to_struct_member(fs, optstr, p);
+
+		/* FS options */
+		if (!rc)
+			rc = mnt_optlist_get_optstr(ol, &p, NULL, MNT_OL_FLTR_UNKNOWN);
+		if (!rc)
+			rc = strdup_to_struct_member(fs, fs_optstr, p);
+
+		/* VFS options */
+		if (!rc)
+			rc = mnt_optlist_get_optstr(ol, &p, mnt_get_builtin_optmap(MNT_LINUX_MAP), 0);
+		if (!rc)
+			rc = strdup_to_struct_member(fs, vfs_optstr, p);
+
+		/* Userspace options */
+		if (!rc)
+			rc = mnt_optlist_get_optstr(ol, &p, mnt_get_builtin_optmap(MNT_USERSPACE_MAP), 0);
+		if (!rc)
+			rc = strdup_to_struct_member(fs, user_optstr, p);
+
+		if (rc) {
+			DBG(FS, ul_debugobj(fs, "sync failed [rc=%d]", rc));
+			return rc;
+		} else {
+			DBG(FS, ul_debugobj(fs, "synced: "
+				"vfs: '%s' fs: '%s' user: '%s', optstr: '%s'",
+				fs->vfs_optstr, fs->fs_optstr, fs->user_optstr, fs->optstr));
+			fs->opts_age = age;
+		}
+	}
+	return 0;
+}
+
+/* If @optlist is not NULL then @fs will read all option strings from @optlist.
+ * It means that mnt_fs_get_*_options() won't be read-only operations. */
+int mnt_fs_follow_optlist(struct libmnt_fs *fs, struct libmnt_optlist *ol)
+{
+	assert(fs);
+
+	if (fs->optlist == ol)
+		return 0;
+	if (fs->optlist)
+		mnt_unref_optlist(fs->optlist);
+
+	fs->opts_age = 0;
+	fs->optlist = ol;
+
+	if (ol)
+		mnt_ref_optlist(ol);
+	return 0;
+}
+
 /**
  * mnt_copy_fs:
  * @dest: destination FS
@@ -175,7 +248,7 @@ static inline int cpy_str_at_offset(void *new, const void *old, size_t offset)
  * set, then the field is NOT overwritten.
  *
  * This function does not copy userdata (se mnt_fs_set_userdata()). A new copy is
- * not linked with any existing mnt_tab.
+ * not linked with any existing mnt_tab or optlist.
  *
  * Returns: @dest or NULL in case of error
  */
@@ -247,13 +320,15 @@ err:
  *
  * Returns: copy of @fs.
  */
-struct libmnt_fs *mnt_copy_mtab_fs(const struct libmnt_fs *fs)
+struct libmnt_fs *mnt_copy_mtab_fs(struct libmnt_fs *fs)
 {
 	struct libmnt_fs *n = mnt_new_fs();
 
 	assert(fs);
 	if (!n)
 		return NULL;
+	if (fs->optlist)
+		sync_opts_from_optlist(fs, fs->optlist);
 
 	if (strdup_between_structs(n, fs, source))
 		goto err;
@@ -552,6 +627,15 @@ int mnt_fs_set_target(struct libmnt_fs *fs, const char *tgt)
 	return strdup_to_struct_member(fs, target, tgt);
 }
 
+int __mnt_fs_set_target_ptr(struct libmnt_fs *fs, char *tgt)
+{
+	assert(fs);
+
+	free(fs->target);
+	fs->target = tgt;
+	return 0;
+}
+
 static int mnt_fs_get_flags(struct libmnt_fs *fs)
 {
 	return fs ? fs->flags : 0;
@@ -600,7 +684,7 @@ int mnt_fs_get_propagation(struct libmnt_fs *fs, unsigned long *flags)
  */
 int mnt_fs_is_kernel(struct libmnt_fs *fs)
 {
-	return mnt_fs_get_flags(fs) & MNT_FS_KERNEL;
+	return mnt_fs_get_flags(fs) & MNT_FS_KERNEL ? 1 : 0;
 }
 
 /**
@@ -611,7 +695,7 @@ int mnt_fs_is_kernel(struct libmnt_fs *fs)
  */
 int mnt_fs_is_swaparea(struct libmnt_fs *fs)
 {
-	return mnt_fs_get_flags(fs) & MNT_FS_SWAP;
+	return mnt_fs_get_flags(fs) & MNT_FS_SWAP ? 1 : 0;
 }
 
 /**
@@ -622,7 +706,7 @@ int mnt_fs_is_swaparea(struct libmnt_fs *fs)
  */
 int mnt_fs_is_pseudofs(struct libmnt_fs *fs)
 {
-	return mnt_fs_get_flags(fs) & MNT_FS_PSEUDO;
+	return mnt_fs_get_flags(fs) & MNT_FS_PSEUDO ? 1 : 0;
 }
 
 /**
@@ -633,7 +717,22 @@ int mnt_fs_is_pseudofs(struct libmnt_fs *fs)
  */
 int mnt_fs_is_netfs(struct libmnt_fs *fs)
 {
-	return mnt_fs_get_flags(fs) & MNT_FS_NET;
+	return mnt_fs_get_flags(fs) & MNT_FS_NET ? 1 : 0;
+}
+
+/**
+ * mnt_fs_is_regularfs:
+ * @fs: filesystem
+ *
+ * Returns: 1 if the filesystem is a regular filesystem (not network or pseudo filesystem).
+ *
+ * Since: 2.38
+ */
+int mnt_fs_is_regularfs(struct libmnt_fs *fs)
+{
+	return !(mnt_fs_is_pseudofs(fs)
+		 || mnt_fs_is_netfs(fs)
+		 || mnt_fs_is_swaparea(fs));
 }
 
 /**
@@ -764,6 +863,8 @@ char *mnt_fs_strdup_options(struct libmnt_fs *fs)
 
 	if (!fs)
 		return NULL;
+	if (fs->optlist)
+		sync_opts_from_optlist(fs, fs->optlist);
 
 	errno = 0;
 	if (fs->optstr)
@@ -788,6 +889,9 @@ char *mnt_fs_strdup_options(struct libmnt_fs *fs)
  */
 const char *mnt_fs_get_options(struct libmnt_fs *fs)
 {
+	if (fs && fs->optlist)
+		sync_opts_from_optlist(fs, fs->optlist);
+
 	return fs ? fs->optstr : NULL;
 }
 
@@ -819,6 +923,12 @@ int mnt_fs_set_options(struct libmnt_fs *fs, const char *optstr)
 
 	if (!fs)
 		return -EINVAL;
+
+	if (fs->optlist) {
+		fs->opts_age = 0;
+		return mnt_optlist_set_optstr(fs->optlist, optstr, NULL);
+	}
+
 	if (optstr) {
 		int rc = mnt_split_optstr(optstr, &u, &v, &f, 0, 0);
 		if (rc)
@@ -866,6 +976,10 @@ int mnt_fs_append_options(struct libmnt_fs *fs, const char *optstr)
 		return -EINVAL;
 	if (!optstr)
 		return 0;
+	if (fs->optlist) {
+		fs->opts_age = 0;
+		return mnt_optlist_append_optstr(fs->optlist, optstr, NULL);
+	}
 
 	rc = mnt_split_optstr(optstr, &u, &v, &f, 0, 0);
 	if (rc)
@@ -909,6 +1023,11 @@ int mnt_fs_prepend_options(struct libmnt_fs *fs, const char *optstr)
 	if (!optstr)
 		return 0;
 
+	if (fs->optlist) {
+		fs->opts_age = 0;
+		return mnt_optlist_prepend_optstr(fs->optlist, optstr, NULL);
+	}
+
 	rc = mnt_split_optstr(optstr, &u, &v, &f, 0, 0);
 	if (rc)
 		return rc;
@@ -929,7 +1048,8 @@ int mnt_fs_prepend_options(struct libmnt_fs *fs, const char *optstr)
 	return rc;
 }
 
-/*
+
+/**
  * mnt_fs_get_fs_options:
  * @fs: fstab/mtab/mountinfo entry pointer
  *
@@ -937,7 +1057,12 @@ int mnt_fs_prepend_options(struct libmnt_fs *fs, const char *optstr)
  */
 const char *mnt_fs_get_fs_options(struct libmnt_fs *fs)
 {
-	return fs ? fs->fs_optstr : NULL;
+	if (!fs)
+		return NULL;
+	if (fs->optlist)
+		sync_opts_from_optlist(fs, fs->optlist);
+
+	return fs->fs_optstr;
 }
 
 /**
@@ -948,7 +1073,12 @@ const char *mnt_fs_get_fs_options(struct libmnt_fs *fs)
  */
 const char *mnt_fs_get_vfs_options(struct libmnt_fs *fs)
 {
-	return fs ? fs->vfs_optstr : NULL;
+	if (!fs)
+		return NULL;
+	if (fs->optlist)
+		sync_opts_from_optlist(fs, fs->optlist);
+
+	return fs->vfs_optstr;
 }
 
 /**
@@ -991,7 +1121,12 @@ char *mnt_fs_get_vfs_options_all(struct libmnt_fs *fs)
  */
 const char *mnt_fs_get_user_options(struct libmnt_fs *fs)
 {
-	return fs ? fs->user_optstr : NULL;
+	if (!fs)
+		return NULL;
+	if (fs->optlist)
+		sync_opts_from_optlist(fs, fs->optlist);
+
+	return fs->user_optstr;
 }
 
 /**
@@ -1281,6 +1416,10 @@ int mnt_fs_get_option(struct libmnt_fs *fs, const char *name,
 
 	if (!fs)
 		return -EINVAL;
+
+	if (fs->optlist)
+		sync_opts_from_optlist(fs, fs->optlist);
+
 	if (fs->fs_optstr)
 		rc = mnt_optstr_get_option(fs->fs_optstr, name, value, valsz);
 	if (rc == 1 && fs->vfs_optstr)
@@ -1354,7 +1493,7 @@ int mnt_fs_append_comment(struct libmnt_fs *fs, const char *comm)
 	if (!fs)
 		return -EINVAL;
 
-	return append_string(&fs->comment, comm);
+	return strappend(&fs->comment, comm);
 }
 
 /**
@@ -1530,6 +1669,10 @@ int mnt_fs_print_debug(struct libmnt_fs *fs, FILE *file)
 {
 	if (!fs || !file)
 		return -EINVAL;
+
+	if (fs->optlist)
+		sync_opts_from_optlist(fs, fs->optlist);
+
 	fprintf(file, "------ fs:\n");
 	fprintf(file, "source: %s\n", mnt_fs_get_source(fs));
 	fprintf(file, "target: %s\n", mnt_fs_get_target(fs));
