@@ -129,10 +129,10 @@ void *scols_cell_get_userdata(struct libscols_cell *ce)
  * @b: pointer to cell
  * @data: unused pointer to private data (defined by API)
  *
- * Compares cells data by strcmp(). The function is designed for
+ * Compares cells data by strcoll(). The function is designed for
  * scols_column_set_cmpfunc() and scols_sort_table().
  *
- * Returns: follows strcmp() return values.
+ * Returns: follows strcoll() return values.
  */
 int scols_cmpstr_cells(struct libscols_cell *a,
 		       struct libscols_cell *b,
@@ -152,7 +152,7 @@ int scols_cmpstr_cells(struct libscols_cell *a,
 		return -1;
 	if (bdata == NULL)
 		return 1;
-	return strcmp(adata, bdata);
+	return strcoll(adata, bdata);
 }
 
 /**
@@ -166,10 +166,13 @@ int scols_cmpstr_cells(struct libscols_cell *a,
  */
 int scols_cell_set_color(struct libscols_cell *ce, const char *color)
 {
-	if (color && isalpha(*color)) {
-		color = color_sequence_from_colorname(color);
-		if (!color)
+	if (color && !color_is_sequence(color)) {
+		char *seq = color_get_sequence(color);
+		if (!seq)
 			return -EINVAL;
+		free(ce->color);
+		ce->color = seq;
+		return 0;
 	}
 	return strdup_to_struct_member(ce, color, color);
 }
