@@ -364,10 +364,13 @@ int scols_line_is_ancestor(struct libscols_line *ln, struct libscols_line *paren
  */
 int scols_line_set_color(struct libscols_line *ln, const char *color)
 {
-	if (color && isalnum(*color)) {
-		color = color_sequence_from_colorname(color);
-		if (!color)
+	if (color && !color_is_sequence(color)) {
+		char *seq = color_get_sequence(color);
+		if (!seq)
 			return -EINVAL;
+		free(ln->color);
+		ln->color = seq;
+		return 0;
 	}
 	return strdup_to_struct_member(ln, color, color);
 }
