@@ -471,7 +471,7 @@ static void write_output(struct logger_ctl *ctl, const char *const msg)
 		union {
 			struct cmsghdr cmh;
 			char   control[CMSG_SPACE(sizeof(struct ucred))];
-		} cbuf;
+		} cbuf = { .control = { 0 } };
 #endif
 
 		/* 4) add extra \n to make sure message is terminated */
@@ -945,8 +945,6 @@ static void logger_open(struct logger_ctl *ctl)
 		ctl->tag = ctl->login = xgetlogin();
 	if (!ctl->tag)
 		ctl->tag = "<someone>";
-
-	generate_syslog_header(ctl);
 }
 
 /* re-open; usually after failed connection */
@@ -996,10 +994,8 @@ static void logger_stdin(struct logger_ctl *ctl)
 {
 	/* note: we re-generate the syslog header for each log message to
 	 * update header timestamps and to reflect possible priority changes.
-	 * The initial header is generated by logger_open().
 	 */
 	int default_priority = ctl->pri;
-	int last_pri = default_priority;
 	char *buf = xmalloc(ctl->max_message_size + 2 + 2);
 	int pri;
 	int c;
@@ -1026,10 +1022,6 @@ static void logger_stdin(struct logger_ctl *ctl)
 			} else
 				ctl->pri = default_priority;
 
-			if (ctl->pri != last_pri) {
-				generate_syslog_header(ctl);
-				last_pri = ctl->pri;
-			}
 			if (c != EOF && c != '\n')
 				c = getchar();
 		}
@@ -1040,8 +1032,10 @@ static void logger_stdin(struct logger_ctl *ctl)
 		}
 		buf[i] = '\0';
 
-		if (i > 0 || !ctl->skip_empty_lines)
+		if (i > 0 || !ctl->skip_empty_lines) {
+			generate_syslog_header(ctl);
 			write_output(ctl, buf);
+		}
 
 		if (c == '\n')	/* discard line terminator */
 			c = getchar();
@@ -1317,12 +1311,14 @@ int main(int argc, char **argv)
 		abort();
 	}
 	logger_open(&ctl);
-	if (0 < argc)
+	if (0 < argc) {
+		generate_syslog_header(&ctl);
 		logger_command_line(&ctl, argv);
-	else
+	} else
 		/* Note. --file <arg> reopens stdin making the below
 		 * function to be used for file inputs. */
 		logger_stdin(&ctl);
+
 	logger_close(&ctl);
 	return EXIT_SUCCESS;
 }
