@@ -39,6 +39,7 @@
 #include "strutils.h"
 #include "carefulputc.h"
 #include "path.h"
+#include "strv.h"
 
 /*
  * since 2.6.18
@@ -109,6 +110,10 @@ static int ncolumns;
 
 struct wd_device {
 	const char	*devpath;
+	struct path_cxt	*sysfs;
+
+	char		*governor;
+	char		**available_governors;
 
 	int		timeout;
 	int		timeleft;
@@ -123,10 +128,19 @@ struct wd_device {
 	unsigned int	has_timeout : 1,
 			has_timeleft : 1,
 			has_pretimeout : 1,
-			has_nowayout : 1;
+			has_nowayout : 1,
+			no_sysfs : 1;
 };
 
 struct wd_control {
+	/* set */
+	int		timeout;			/* --settimeout */
+	int		pretimeout;			/* --setpretimeout */
+	const char      *governor;			/* --setpregovernor */
+	unsigned int	set_timeout : 1,
+			set_pretimeout : 1;
+
+	/* output */
 	unsigned int	show_oneline : 1,
 			show_raw : 1,
 			hide_headings : 1,
@@ -135,6 +149,10 @@ struct wd_control {
 			hide_timeouts : 1;
 };
 
+#define want_set(_ctl)		((_ctl)->set_timeout \
+				  || (_ctl)->set_pretimeout \
+				  || (_ctl)->governor)
+
 /* converts flag name to flag bit */
 static long name2bit(const char *name, size_t namesz)
 {
@@ -216,6 +234,8 @@ static void __attribute__((__noreturn__)) usage(void)
 		" -n, --noheadings       don't print headings for flags table\n"
 		" -O, --oneline          print all information on one line\n"
 		" -o, --output <list>    output columns of the flags\n"
+		" -p, --setpretimeout <sec> set watchdog pre-timeout\n"
+		" -g, --setpregovernor <name> set pre-timeout governor\n"
 		" -r, --raw              use raw output format for flags table\n"
 		" -T, --notimeouts       don't print watchdog timeouts\n"
 		" -s, --settimeout <sec> set watchdog timeout\n"
@@ -239,6 +259,36 @@ static void __attribute__((__noreturn__)) usage(void)
 	exit(EXIT_SUCCESS);
 }
 
+static struct path_cxt *get_sysfs(struct wd_device *wd)
+{
+	struct path_cxt *sys;
+	struct stat st;
+
+	if (wd->no_sysfs)
+		return NULL;
+	if (wd->sysfs)
+		return wd->sysfs;
+	if (stat(wd->devpath, &st) != 0)
+		goto nosysfs;
+
+	sys = ul_new_path(_PATH_SYS_DEVCHAR "/%u:%u",
+			major(st.st_rdev), minor(st.st_rdev));
+	if (!sys)
+		return NULL;
+
+	if (ul_path_get_dirfd(sys) < 0)
+		goto nosysfs;		/* device not in /sys */
+
+	if (ul_path_access(sys, F_OK, "identity") != 0)
+		goto nosysfs;		/* no info in /sys (old miscdev?) */
+
+	wd->sysfs = sys;
+	return sys;
+nosysfs:
+	wd->no_sysfs = 1;
+	return NULL;
+}
+
 static void add_flag_line(struct libscols_table *table, struct wd_device *wd, const struct wdflag *fl)
 {
 	int i;
@@ -334,19 +384,25 @@ done:
 	scols_unref_table(table);
 	return rc;
 }
+
 /*
  * Warning: successfully opened watchdog has to be properly closed with magic
  * close character otherwise the machine will be rebooted!
  *
  * Don't use err() or exit() here!
  */
-static int set_watchdog(struct wd_device *wd, int timeout)
+static int set_watchdog(struct wd_control *ctl, struct wd_device *wd)
 {
 	int fd;
 	sigset_t sigs, oldsigs;
 	int rc = 0;
 
+	assert(wd);
 	assert(wd->devpath);
+	assert(ctl);
+
+	if (!ctl->set_timeout && !ctl->set_timeout)
+		goto sysfs_only;
 
 	sigemptyset(&oldsigs);
 	sigfillset(&sigs);
@@ -377,16 +433,43 @@ static int set_watchdog(struct wd_device *wd, int timeout)
 		 * the machine might end up rebooting. */
 	}
 
-	if (ioctl(fd, WDIOC_SETTIMEOUT, &timeout) != 0) {
-		rc = errno;
-		warn(_("cannot set timeout for %s"), wd->devpath);
+	if (ctl->set_timeout) {
+		if (ioctl(fd, WDIOC_SETTIMEOUT, &ctl->timeout) != 0) {
+			rc += errno;
+			warn(_("cannot set timeout for %s"), wd->devpath);
+		} else
+			printf(P_("Timeout has been set to %d second.\n",
+				  "Timeout has been set to %d seconds.\n",
+				  ctl->timeout), ctl->timeout);
+	}
+
+	if (ctl->set_pretimeout) {
+		if (ioctl(fd, WDIOC_SETPRETIMEOUT, &ctl->pretimeout) != 0) {
+			rc += errno;
+			warn(_("cannot set pretimeout for %s"), wd->devpath);
+		} else
+			printf(P_("Pre-timeout has been set to %d second.\n",
+				  "Pre-timeout has been set to %d seconds.\n",
+				  ctl->pretimeout), ctl->pretimeout);
 	}
 
 	if (close(fd))
 		warn(_("write failed"));
+
 	sigprocmask(SIG_SETMASK, &oldsigs, NULL);
-	printf(P_("Timeout has been set to %d second.\n",
-		  "Timeout has been set to %d seconds.\n", timeout), timeout);
+
+sysfs_only:
+	if (ctl->governor) {
+		struct path_cxt *sys = get_sysfs(wd);
+		int xrc;
+
+		xrc = !sys ? errno :
+			     ul_path_write_string(sys, ctl->governor,
+						"pretimeout_governor");
+		if (xrc)
+			warn(_("cannot set pre-timeout governor"));
+		rc += xrc;
+	}
 
 	return rc;
 }
@@ -419,6 +502,13 @@ static int read_watchdog_from_device(struct wd_device *wd)
 		ioctl(fd, WDIOC_GETSTATUS, &wd->status);
 		ioctl(fd, WDIOC_GETBOOTSTATUS, &wd->bstatus);
 
+		/*
+		 * Sometimes supported options like WDIOF_CARDRESET are missing from
+		 * ident.options, add anything set in status/bstatus to ident.options.
+		 */
+		wd->ident.options |= wd->status;
+		wd->ident.options |= wd->bstatus;
+
 		if (ioctl(fd, WDIOC_GETTIMEOUT, &wd->timeout) >= 0)
 			wd->has_timeout = 1;
 		if (ioctl(fd, WDIOC_GETPRETIMEOUT, &wd->pretimeout) >= 0)
@@ -449,29 +539,19 @@ static int read_watchdog_from_device(struct wd_device *wd)
 	return 0;
 }
 
+
 /* Returns: <0 error, 0 success, 1 unssuported */
 static int read_watchdog_from_sysfs(struct wd_device *wd)
 {
 	struct path_cxt *sys;
-	struct stat st;
-	int rc;
 
-	rc = stat(wd->devpath, &st);
-	if (rc != 0)
-		return rc;
-
-	sys = ul_new_path(_PATH_SYS_DEVCHAR "/%u:%u",
-			major(st.st_rdev), minor(st.st_rdev));
+	sys = get_sysfs(wd);
 	if (!sys)
-		return -ENOMEM;
-
-	if (ul_path_get_dirfd(sys) < 0)
-		goto nosysfs;		/* device not in /sys */
-
-	if (ul_path_access(sys, F_OK, "identity") != 0)
-		goto nosysfs;		/* no info in /sys (old miscdev?) */
+		return 1;
 
 	ul_path_read_buffer(sys, (char *) wd->ident.identity, sizeof(wd->ident.identity), "identity");
+	ul_path_read_u32(sys, &wd->ident.firmware_version, "fw_version");
+	ul_path_scanf(sys, "options", "%x", &wd->ident.options);
 
 	ul_path_scanf(sys, "status", "%x", &wd->status);
 	ul_path_read_u32(sys, &wd->bstatus, "bootstatus");
@@ -485,11 +565,37 @@ static int read_watchdog_from_sysfs(struct wd_device *wd)
 	if (ul_path_read_s32(sys, &wd->timeleft, "timeleft") == 0)
 		wd->has_timeleft = 1;
 
-	ul_unref_path(sys);
 	return 0;
-nosysfs:
-	ul_unref_path(sys);
-	return 1;
+}
+
+static int read_governors(struct wd_device *wd)
+{
+	struct path_cxt *sys;
+	FILE *f;
+
+	sys = get_sysfs(wd);
+	if (!sys)
+		return 1;
+
+	f = ul_path_fopen(sys, "r", "pretimeout_available_governors");
+	if (f) {
+		char *line = NULL;
+		size_t dummy = 0;
+		ssize_t sz;
+
+		while ((sz = getline(&line, &dummy, f)) >= 0) {
+			if (rtrim_whitespace((unsigned char *) line) == 0)
+				continue;
+			strv_consume(&wd->available_governors, line);
+			dummy = 0;
+			line = NULL;
+		}
+		free(line);
+		fclose(f);
+	}
+
+	ul_path_read_string(sys, &wd->governor,  "pretimeout_governor");
+	return 0;
 }
 
 static int read_watchdog(struct wd_device *wd)
@@ -504,6 +610,7 @@ static int read_watchdog(struct wd_device *wd)
 		return -1;
 	}
 
+	read_governors(wd);
 	return 0;
 }
 
@@ -512,12 +619,26 @@ static void show_timeouts(struct wd_device *wd)
 	if (wd->has_timeout)
 		printf(P_("%-14s %2i second\n", "%-14s %2i seconds\n", wd->timeout),
 			  _("Timeout:"), wd->timeout);
-	if (wd->has_pretimeout)
-		printf(P_("%-14s %2i second\n", "%-14s %2i seconds\n", wd->pretimeout),
-			  _("Pre-timeout:"), wd->pretimeout);
 	if (wd->has_timeleft)
 		printf(P_("%-14s %2i second\n", "%-14s %2i seconds\n", wd->timeleft),
 			  _("Timeleft:"), wd->timeleft);
+	if (wd->has_pretimeout)
+		printf(P_("%-14s %2i second\n", "%-14s %2i seconds\n", wd->pretimeout),
+			  _("Pre-timeout:"), wd->pretimeout);
+}
+
+static void show_governors(struct wd_device *wd)
+{
+	if (wd->governor)
+		printf(_("%-14s %s\n"), _("Pre-timeout governor:"), wd->governor);
+	if (wd->available_governors) {
+		char *tmp = strv_join(wd->available_governors, " ");
+
+		if (tmp)
+			printf(_("%-14s %s\n"),
+				_("Available pre-timeout governors:"), tmp);
+		free(tmp);
+	}
 }
 
 static void print_oneline(struct wd_control *ctl, struct wd_device *wd, uint32_t wanted)
@@ -583,6 +704,8 @@ static void print_device(struct wd_control *ctl, struct wd_device *wd, uint32_t
 	if (!ctl->hide_timeouts)
 		show_timeouts(wd);
 
+	show_governors(wd);
+
 	if (!ctl->hide_flags)
 		show_flags(ctl, wd, wanted);
 }
@@ -592,8 +715,7 @@ int main(int argc, char *argv[])
 	struct wd_device wd;
 	struct wd_control ctl = { .hide_headings = 0 };
 	int c, res = EXIT_SUCCESS, count = 0;
-	uint32_t wanted = 0;
-	int timeout = 0;
+	unsigned long wanted = 0;
 	const char *dflt_device = NULL;
 
 	static const struct option long_opts[] = {
@@ -605,6 +727,8 @@ int main(int argc, char *argv[])
 		{ "noident",	no_argument,       NULL, 'I' },
 		{ "notimeouts", no_argument,       NULL, 'T' },
 		{ "settimeout", required_argument, NULL, 's' },
+		{ "setpretimeout", required_argument, NULL, 'p' },
+		{ "setpregovernor", required_argument, NULL, 'g' },
 		{ "output",     required_argument, NULL, 'o' },
 		{ "oneline",    no_argument,       NULL, 'O' },
 		{ "raw",        no_argument,       NULL, 'r' },
@@ -624,7 +748,7 @@ int main(int argc, char *argv[])
 	close_stdout_atexit();
 
 	while ((c = getopt_long(argc, argv,
-				"d:f:hFnITo:s:OrVx", long_opts, NULL)) != -1) {
+				"d:f:g:hFnITp:o:s:OrVx", long_opts, NULL)) != -1) {
 
 		err_exclusive_options(c, long_opts, excl, excl_st);
 
@@ -637,15 +761,23 @@ int main(int argc, char *argv[])
 				return EXIT_FAILURE;
 			break;
 		case 's':
-			timeout = strtos32_or_err(optarg, _("invalid timeout argument"));
+			ctl.timeout = strtos32_or_err(optarg, _("invalid timeout argument"));
+			ctl.set_timeout = 1;
+			break;
+		case 'p':
+			ctl.pretimeout = strtos32_or_err(optarg, _("invalid pretimeout argument"));
+			ctl.set_pretimeout = 1;
 			break;
 		case 'f':
-			if (string_to_bitmask(optarg, (unsigned long *) &wanted, name2bit) != 0)
+			if (string_to_bitmask(optarg, &wanted, name2bit) != 0)
 				return EXIT_FAILURE;
 			break;
 		case 'F':
 			ctl.hide_flags = 1;
 			break;
+		case 'g':
+			ctl.governor = optarg;
+			break;
 		case 'I':
 			ctl.hide_ident = 1;
 			break;
@@ -700,8 +832,8 @@ int main(int argc, char *argv[])
 			fputc('\n', stdout);
 		count++;
 
-		if (timeout) {
-			rc = set_watchdog(&wd, timeout);
+		if (want_set(&ctl)) {
+			rc = set_watchdog(&ctl, &wd);
 			if (rc) {
 				res = EXIT_FAILURE;
 			}
@@ -714,7 +846,7 @@ int main(int argc, char *argv[])
 		}
 
 		print_device(&ctl, &wd, wanted);
-
+		ul_unref_path(wd.sysfs);
 	} while (optind < argc);
 
 	return res;
